@@ -22,12 +22,16 @@ import java.security.KeyStore;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Dictionary;
+import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Properties;
+import java.util.zip.Deflater;
 
+import org.apache.felix.http.base.internal.HttpConfig;
 import org.apache.felix.http.base.internal.logger.SystemLogger;
+import org.eclipse.jetty.server.CustomRequestLog;
+import org.eclipse.jetty.server.handler.gzip.GzipHandler;
 import org.osgi.framework.BundleContext;
 
 public final class JettyConfig
@@ -137,13 +141,13 @@ public final class JettyConfig
 
     /** Felix specific property to control whether to enable Proxy/Load Balancer Connection */
     public static final String FELIX_PROXY_LOAD_BALANCER_CONNECTION_ENABLE = "org.apache.felix.proxy.load.balancer.connection.enable";
-    
+
     /** Felix specific property to configure the session cookie httpOnly flag */
     public static final String FELIX_JETTY_SESSION_COOKIE_HTTP_ONLY = "org.apache.felix.https.jetty.session.cookie.httpOnly";
-    
+
     /** Felix specific property to configure the session cookie secure flag */
     public static final String FELIX_JETTY_SESSION_COOKIE_SECURE = "org.apache.felix.https.jetty.session.cookie.secure";
-    
+
     /** Felix specific property to configure session id path parameter*/
     public static final String FELIX_JETTY_SERVLET_SESSION_ID_PATH_PARAMETER_NAME = "org.eclipse.jetty.servlet.SessionIdPathParameterName";
 
@@ -162,6 +166,108 @@ public final class JettyConfig
     /**  Felix specific property to configure session max age */
     public static final String FELIX_JETTY_SERVLET_SESSION_MAX_AGE = "org.eclipse.jetty.servlet.MaxAge";
 
+    /** Felix specific property to configure session scavenging interval in Seconds */
+    public static final String FELIX_JETTY_SESSION_SCAVENGING_INTERVAL = "org.eclipse.jetty.servlet.SessionScavengingInterval";
+
+    /** Felix specific property to set HTTP instance name. */
+    public static final String FELIX_HTTP_SERVICE_NAME = "org.apache.felix.http.name";
+
+    /** Felix specific property to configure a filter for RequestLog services */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILTER = "org.apache.felix.http.requestlog.filter";
+
+    /** Felix specific property to enable request logging through SLF4J name "org.apache.felix.http" (with level INFO) */
+    public static final String FELIX_HTTP_REQUEST_LOG_OSGI_ENABLE = "org.apache.felix.http.requestlog.osgi.enable";
+
+    /** Felix specific property to specify the published "name" property of the SLF4J Service-based Request Log service. Allows server configs to filter on specific log services. */
+    public static final String FELIX_HTTP_REQUEST_LOG_OSGI_SERVICE_NAME = "org.apache.felix.http.requestlog.osgi.name";
+
+    /** Felix specific property to specify the logger name of the SLF4J Service-based Request Log service. Log entries are emitted with level INFO. */
+    public static final String FELIX_HTTP_REQUEST_LOG_OSGI_LOGGER_NAME = "org.apache.felix.http.requestlog.osgi.logger.name";
+
+    /** Felix specific property to control the format of the log messages generated by the OSGi Log Service-based request log. */
+    public static final String FELIX_HTTP_REQUEST_LOG_FORMAT = "org.apache.felix.http.requestlog.osgi.format";
+
+    /** Felix specific property to enable request logging to a file and provide the path to that file. Default is null meaning that the file log is disabled. */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_PATH = "org.apache.felix.http.requestlog.file.path";
+
+    /** Felix specific property to specify the published "name" property of the file-based RequestLog service. Allows server configs to filter on specific log services. */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_SERVICE_NAME = "org.apache.felix.http.requestlog.file.name";
+
+    /** Felix specific property to enable file request logging to be asynchronous */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_ASYNC = "org.apache.felix.http.requestlog.file.async";
+
+    /** Felix specific property to enable request logging to append to the log file rather than overwriting */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_APPEND = "org.apache.felix.http.requestlog.file.append";
+
+    /** Felix specific property to specify the number of days the request log file is retained */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_RETAIN_DAYS = "org.apache.felix.http.requestlog.file.retaindays";
+
+    /** Felix specific property to specify the date format in request log file names */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_FILENAME_DATE_FORMAT = "org.apache.felix.http.requestlog.file.dateformat";
+
+    /** Felix specific property to ignore matching paths in the request log file */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_IGNORE_PATHS = "org.apache.felix.http.requestlog.file.ignorepaths";
+
+    /** Felix specific property to specify the output format for the request log file */
+    public static final String FELIX_HTTP_REQUEST_LOG_FILE_FORMAT = "org.apache.felix.http.requestlog.file.format";
+
+    /** Felix specific property to define custom properties for the http runtime service. */
+    public static final String FELIX_CUSTOM_HTTP_RUNTIME_PROPERTY_PREFIX = "org.apache.felix.http.runtime.init.";
+
+    /** Felix specific property to specify whether the server should collect statistics information (defaults to false) */
+    public static final String FELIX_JETTY_STATISTICS_HANDLER_ENABLE = "org.apache.felix.jetty.statisticshandler.enable";
+
+    /** Felix specific property to specify whether the server should use a server-wide gzip handler (defaults to true) */
+    public static final String FELIX_JETTY_GZIP_HANDLER_ENABLE = "org.apache.felix.jetty.gziphandler.enable";
+
+    /** Felix specific property to specify the minimum response size to trigger dynamic compression */
+    public static final String FELIX_JETTY_GZIP_MIN_GZIP_SIZE = "org.apache.felix.jetty.gzip.minGzipSize";
+
+    /** Felix specific property to specify the size in bytes of the buffer to inflate compressed request, or 0 for no inflation. */
+    public static final String FELIX_JETTY_GZIP_INFLATE_BUFFER_SIZE = "org.apache.felix.jetty.gzip.inflateBufferSize";
+
+    /** Felix specific property to specify the {@link Deflater} flush mode to use. */
+    public static final String FELIX_JETTY_GZIP_SYNC_FLUSH = "org.apache.felix.jetty.gzip.syncFlush";
+
+    /** Felix specific property to specify the methods to include in compression */
+    public static final String FELIX_JETTY_GZIP_INCLUDED_METHODS = "org.apache.felix.jetty.gzip.includedMethods";
+
+    /** Felix specific property to specify the methods to exclude from compression */
+    public static final String FELIX_JETTY_GZIP_EXCLUDED_METHODS = "org.apache.felix.jetty.gzip.excludedMethods";
+
+    /** Felix specific property to specify the path specs to include. Inclusion takes precedence over exclusion. */
+    public static final String FELIX_JETTY_GZIP_INCLUDED_PATHS = "org.apache.felix.jetty.gzip.includedPaths";
+
+    /** Felix specific property to specify the path specs to exclude. */
+    public static final String FELIX_JETTY_GZIP_EXCLUDED_PATHS = "org.apache.felix.jetty.gzip.excludedPaths";
+
+    /** Felix specific property to specify the included mime types. Inclusion takes precedence over exclusion. */
+    public static final String FELIX_JETTY_GZIP_INCLUDED_MIME_TYPES = "org.apache.felix.jetty.gzip.includedMimeTypes";
+
+    /** Felix specific property to specify the excluded mime types. */
+    public static final String FELIX_JETTY_GZIP_EXCLUDED_MIME_TYPES = "org.apache.felix.jetty.gzip.excludedMimeTypes";
+
+    /** Felix specific property to specify the stop timeout of the jetty server */
+    public static final String FELIX_JETTY_STOP_TIMEOUT = "org.apache.felix.jetty.stopTimeout";
+
+    /** Felix specific property to control whether to enable HTTP/2. */
+    public static final String FELIX_HTTP2_ENABLE = "org.apache.felix.http2.enable";
+
+    /** Felix specific property to specify the max number of concurrent streams per connection  */
+    public static final String FELIX_JETTY_HTTP2_MAX_CONCURRENT_STREAMS = "org.apache.felix.jetty.http2.maxConcurrentStreams";
+
+    /** Felix specific property to specify the initial stream receive window (client to server)  */
+    public static final String FELIX_JETTY_HTTP2_INITIAL_STREAM_RECV_WINDOW = "org.apache.felix.jetty.http2.initialStreamRecvWindow";
+
+    /** Felix specific property to specify the initial session receive window (client to server)  */
+    public static final String FELIX_JETTY_HTTP2_INITIAL_SESSION_RECV_WINDOW = "org.apache.felix.jetty.http2.initialSessionRecvWindow";
+
+    /** Felix specific property to specify the ALPN protocols to consider  */
+    public static final String FELIX_JETTY_ALPN_PROTOCOLS = "org.apache.felix.jetty.alpn.protocols";
+
+    /** Felix specific property to specify the default protocol when negotiation fails  */
+    public static final String FELIX_JETTY_ALPN_DEFAULT_PROTOCOL = "org.apache.felix.jetty.alpn.defaultProtocol";
+
     private static String validateContextPath(String ctxPath)
     {
         // undefined, empty, or root context path
@@ -193,9 +299,13 @@ public final class JettyConfig
      * This map is indexed by String objects (the property names) and
      * the values are just objects as provided by the configuration.
      */
-    private volatile Dictionary config;
+    private volatile Dictionary<String, ?> config;
 
-    public JettyConfig(BundleContext context)
+    private volatile Integer httpPort;
+
+    private volatile Integer httpsPort;
+
+    public JettyConfig(final BundleContext context)
     {
         this.context = context;
         reset();
@@ -254,12 +364,18 @@ public final class JettyConfig
 
     public int getHttpPort()
     {
-        return determinePort(String.valueOf(getProperty(HTTP_PORT)), 8080);
+        if (httpPort == null) {
+            httpPort = determinePort(String.valueOf(getProperty(HTTP_PORT)), 8080);
+        }
+        return httpPort;
     }
 
     public int getHttpsPort()
     {
-        return determinePort(String.valueOf(getProperty(HTTPS_PORT)), 8443);
+        if (httpsPort == null) {
+            httpsPort = determinePort(String.valueOf(getProperty(HTTPS_PORT)), 8443);
+        }
+        return httpsPort;
     }
 
     public int getHttpTimeout()
@@ -282,6 +398,16 @@ public final class JettyConfig
         return parseInt(getProperty(name, null), defValue);
     }
 
+    /**
+     * Returns the named generic configuration property from the
+     * configuration or the bundle context. If neither property is defined
+     * return the defValue.
+     */
+    public long getLongProperty(String name, long defValue)
+    {
+        return parseLong(getProperty(name, null), defValue);
+    }
+
     public String getKeyPassword()
     {
         return getProperty(FELIX_KEYSTORE_KEY_PASSWORD, this.context.getProperty(OSCAR_KEYSTORE_KEY_PASSWORD));
@@ -405,6 +531,35 @@ public final class JettyConfig
         return useHttps && getHttpsPort() > 0;
     }
 
+    /**
+     * Returns <code>true</code> if HTTP/2 is configured to be used (
+     * {@link #FELIX_HTTP2_ENABLE})
+     */
+    public boolean isUseHttp2()
+    {
+        return getBooleanProperty(FELIX_HTTP2_ENABLE, false);
+    }
+
+    public int getHttp2MaxConcurrentStreams() {
+        return getIntProperty(FELIX_JETTY_HTTP2_MAX_CONCURRENT_STREAMS, 128);
+    }
+
+    public int getHttp2InitialStreamRecvWindow() {
+        return getIntProperty(FELIX_JETTY_HTTP2_INITIAL_STREAM_RECV_WINDOW, 524288);
+    }
+
+    public int getHttp2InitialSessionRecvWindow() {
+        return getIntProperty(FELIX_JETTY_HTTP2_INITIAL_SESSION_RECV_WINDOW, 1048576);
+    }
+
+    public String[] getAlpnProtocols() {
+        return getStringArrayProperty(FELIX_JETTY_ALPN_PROTOCOLS, new String[] {"h2", "http/1.1"} );
+    }
+
+    public String getAlpnDefaultProtocol() {
+        return getProperty(FELIX_JETTY_ALPN_DEFAULT_PROTOCOL, "http/1.1");
+    }
+
     public boolean isProxyLoadBalancerConnection()
     {
         return getBooleanProperty(FELIX_PROXY_LOAD_BALANCER_CONNECTION_ENABLE, false);
@@ -414,6 +569,111 @@ public final class JettyConfig
         return getBooleanProperty(FELIX_JETTY_RENEGOTIATION_ALLOWED, false);
     }
 
+    public String getHttpServiceName()
+    {
+    	return (String) getProperty(FELIX_HTTP_SERVICE_NAME);
+    }
+
+    public String getRequestLogFilter() {
+        return getProperty(FELIX_HTTP_REQUEST_LOG_FILTER, null);
+    }
+
+    public boolean isRequestLogOSGiEnabled() {
+        return getBooleanProperty(FELIX_HTTP_REQUEST_LOG_OSGI_ENABLE, false);
+    }
+
+    public String getRequestLogOSGiServiceName() {
+        return (String) getProperty(FELIX_HTTP_REQUEST_LOG_OSGI_SERVICE_NAME);
+    }
+
+    public String getRequestLogOsgiSlf4JLoggerName() {
+        return getProperty(FELIX_HTTP_REQUEST_LOG_OSGI_LOGGER_NAME, SystemLogger.LOGGER.getName());
+    }
+
+    public String getRequestLogOSGiFormat() {
+        return getProperty(FELIX_HTTP_REQUEST_LOG_FORMAT, CustomRequestLog.EXTENDED_NCSA_FORMAT);
+    }
+
+    public String getRequestLogFilePath() {
+        return getProperty(FELIX_HTTP_REQUEST_LOG_FILE_PATH, null);
+    }
+
+    public String getRequestLogFileServiceName() {
+        return getProperty(FELIX_HTTP_REQUEST_LOG_FILE_SERVICE_NAME, "file");
+    }
+
+    public boolean isRequestLogFileAsync() {
+        return getBooleanProperty(FELIX_HTTP_REQUEST_LOG_FILE_ASYNC, false);
+    }
+
+    public boolean isRequestLogFileAppend() {
+        return getBooleanProperty(FELIX_HTTP_REQUEST_LOG_FILE_APPEND, true);
+    }
+
+    public int getRequestLogFileRetainDays() {
+        return getIntProperty(FELIX_HTTP_REQUEST_LOG_FILE_RETAIN_DAYS, 31);
+    }
+
+    public String getRequestLogFileFormat() {
+        return getProperty(FELIX_HTTP_REQUEST_LOG_FILE_FORMAT, CustomRequestLog.NCSA_FORMAT);
+    }
+
+    public String getRequestLogFilenameDateFormat() {
+        return getProperty(FELIX_HTTP_REQUEST_LOG_FILE_FILENAME_DATE_FORMAT, null);
+    }
+
+    public String[] getRequestLogFileIgnorePaths() {
+        return getStringArrayProperty(FELIX_HTTP_REQUEST_LOG_FILE_IGNORE_PATHS, new String[0]);
+    }
+
+    public boolean isStatisticsHandlerEnabled() {
+        return getBooleanProperty(FELIX_JETTY_STATISTICS_HANDLER_ENABLE, false);
+    }
+
+    public boolean isGzipHandlerEnabled() {
+        return getBooleanProperty(FELIX_JETTY_GZIP_HANDLER_ENABLE, false);
+    }
+
+    public int getGzipMinGzipSize() {
+        return getIntProperty(FELIX_JETTY_GZIP_MIN_GZIP_SIZE, GzipHandler.DEFAULT_MIN_GZIP_SIZE);
+    }
+
+    public int getGzipInflateBufferSize() {
+        return getIntProperty(FELIX_JETTY_GZIP_INFLATE_BUFFER_SIZE, -1);
+    }
+
+    public boolean isGzipSyncFlush() {
+        return getBooleanProperty(FELIX_JETTY_GZIP_SYNC_FLUSH, false);
+    }
+
+    public String[] getGzipIncludedMethods() {
+        return getStringArrayProperty(FELIX_JETTY_GZIP_INCLUDED_METHODS, new String[0]);
+    }
+
+    public String[] getGzipExcludedMethods() {
+        return getStringArrayProperty(FELIX_JETTY_GZIP_EXCLUDED_METHODS, new String[0]);
+    }
+
+    public String[] getGzipIncludedPaths() {
+        return getStringArrayProperty(FELIX_JETTY_GZIP_INCLUDED_PATHS, new String[0]);
+    }
+
+    public String[] getGzipExcludedPaths() {
+        return getStringArrayProperty(FELIX_JETTY_GZIP_EXCLUDED_PATHS, new String[0]);
+    }
+
+    public String[] getGzipIncludedMimeTypes() {
+        return getStringArrayProperty(FELIX_JETTY_GZIP_INCLUDED_MIME_TYPES, new String[0]);
+    }
+
+    public String[] getGzipExcludedMimeTypes() {
+        return getStringArrayProperty(FELIX_JETTY_GZIP_EXCLUDED_MIME_TYPES, new String[0]);
+    }
+
+    public long getStopTimeout() {
+        return getLongProperty(FELIX_JETTY_STOP_TIMEOUT, -1l);
+    }
+
     public void reset()
     {
         update(null);
@@ -425,6 +685,32 @@ public final class JettyConfig
         props.put(HTTPS_PORT, Integer.toString(getHttpsPort()));
         props.put(FELIX_HTTP_ENABLE, Boolean.toString(isUseHttp()));
         props.put(FELIX_HTTPS_ENABLE, Boolean.toString(isUseHttps()));
+        if (getHttpServiceName() != null)
+        {
+			props.put(FELIX_HTTP_SERVICE_NAME, getHttpServiceName());
+        }
+
+        props.put(HttpConfig.PROP_INVALIDATE_SESSION, getBooleanProperty(HttpConfig.PROP_INVALIDATE_SESSION,
+                HttpConfig.DEFAULT_INVALIDATE_SESSION));
+        props.put(HttpConfig.PROP_UNIQUE_SESSION_ID, getBooleanProperty(HttpConfig.PROP_UNIQUE_SESSION_ID,
+                HttpConfig.DEFAULT_UNIQUE_SESSION_ID));
+        props.put(HttpConfig.PROP_CONTAINER_ADDED_ATTRIBUTE, getStringArrayProperty(HttpConfig.PROP_CONTAINER_ADDED_ATTRIBUTE,
+                new String[] {"org.eclipse.jetty.security.sessionCreatedSecure"}));
+
+        addCustomServiceProperties(props);
+    }
+
+    private void addCustomServiceProperties(final Hashtable<String, Object> props)
+    {
+        final Enumeration<String> keys = this.config.keys();
+        while(keys.hasMoreElements())
+        {
+            final String key = keys.nextElement();
+            if (key.startsWith(FELIX_CUSTOM_HTTP_RUNTIME_PROPERTY_PREFIX))
+            {
+                props.put(key.substring(FELIX_CUSTOM_HTTP_RUNTIME_PROPERTY_PREFIX.length()), this.config.get(key));
+            }
+        }
     }
 
     /**
@@ -433,15 +719,19 @@ public final class JettyConfig
      * @param props the dictionary with the new configuration values, can be <code>null</code> to reset this configuration to its defaults.
      * @return <code>true</code> if the configuration was updated due to a changed value, or <code>false</code> if no change was found.
      */
-    public boolean update(Dictionary props)
+    public boolean update(Dictionary<String, ?> props)
     {
         if (props == null)
         {
-            props = new Properties();
+            props = new Hashtable<>();
         }
 
+        // clear cached ports
+        this.httpPort = null;
+        this.httpsPort = null;
+
         // FELIX-4312 Check whether there's something changed in our configuration...
-        Dictionary currentConfig = this.config;
+        Dictionary<String, ?> currentConfig = this.config;
         if (currentConfig == null || !props.equals(currentConfig))
         {
             this.config = props;
@@ -557,7 +847,7 @@ public final class JettyConfig
         }
         catch (IOException e)
         {
-            SystemLogger.debug("Unable to bind to port: " + i);
+            SystemLogger.LOGGER.debug("Unable to bind to port: {}", i);
         }
         finally
         {
@@ -566,9 +856,9 @@ public final class JettyConfig
         return port;
     }
 
-    private Object getProperty(String name)
+    private Object getProperty(final String name)
     {
-        Dictionary conf = this.config;
+        Dictionary<String, ?> conf = this.config;
         Object value = (conf != null) ? conf.get(name) : null;
         if (value == null)
         {
@@ -596,7 +886,7 @@ public final class JettyConfig
         else if (value instanceof String[])
         {
             final String[] stringArr = (String[]) value;
-            final List<String> list = new ArrayList<String>();
+            final List<String> list = new ArrayList<>();
             for (final String stringVal : stringArr)
             {
                 if (stringVal.trim().length() > 0)
@@ -611,7 +901,7 @@ public final class JettyConfig
         }
         else if (value instanceof Collection)
         {
-            final ArrayList<String> conv = new ArrayList<String>();
+            final ArrayList<String> conv = new ArrayList<>();
             for (Iterator<?> vi = ((Collection<?>) value).iterator(); vi.hasNext();)
             {
                 Object object = vi.next();
@@ -639,4 +929,16 @@ public final class JettyConfig
             return dflt;
         }
     }
+
+    private long parseLong(String value, long dflt)
+    {
+        try
+        {
+            return Long.parseLong(value);
+        }
+        catch (NumberFormatException e)
+        {
+            return dflt;
+        }
+    }
 }
